
import settings as test_settings
from django.conf import settings
settings.configure(**test_settings.__dict__)

print ""
print "ROOT_URLCONF = %s" % settings.ROOT_URLCONF
print ""

if __name__ == "__main__":
    
    from django.core.urlresolvers import reverse
    from django.core.urlresolvers import NoReverseMatch
    
    try:
        print "REVERSE: %s" % reverse('testapp:views:entry',
            current_app='testapp',
            kwargs=dict(blog='ost-blog', entry='yo-dogg-i-heard-you-liked-blogs'))
    except NoReverseMatch, err:
        print "NoReverseMatch: %s" % err
    
    try:
        print "REVERSE: %s" % reverse('testapp:testapp:entry',
            current_app='testapp',
            kwargs=dict(blog='ost-blog', entry='yo-dogg-i-heard-you-liked-blogs'))
    except NoReverseMatch, err:
        print "NoReverseMatch: %s" % err
    
    from test import urls
    from url_namespaces import modulize
    try:
        print "REVERSE: %s" % reverse('testapp:entry',
            current_app='testapp',
            kwargs=dict(blog='ost-blog', entry='yo-dogg-i-heard-you-liked-blogs'))
    except NoReverseMatch, err:
        print "NoReverseMatch: %s" % err
    
    #pprint(modulize(ns.grafs).urlpatterns)





import re
import sys
import types
from pprint import pprint

from django.conf.urls import url
from django.conf.urls import patterns
from django.conf.urls import include as django_include
from django.core.urlresolvers import reverse_lazy, reverse
from django.views.generic.simple import redirect_to
from django.utils.importlib import import_module
from django.utils.functional import memoize

CACHE = {}

class memo(object):
    def __init__(self, cache, num_args=2):
        self.cache = cache
        self.num_args = num_args
    def __call__(self, f):
        return memoize(f, self.cache, self.num_args)

class ClassIncludeError(Exception):
    pass

def modulize(ns):
    ns_modname = "__url_namespaces__.%s" % ns.namespace.replace(':', '_')
    ns_module = types.ModuleType(ns_modname, "Dynamic URLConf Module")
    ns_module.__dict__.update({
        'urlpatterns': ns._evaluate(), })
    print "ns_modname = %s" % ns_modname
    sys.modules.update({
        ns_modname: ns_module, })
    return ns_module

def include_class(path):
    path_bits = path.split('.') # Cut off the class name at the end.
    class_name = path_bits.pop()
    module_path = '.'.join(path_bits)
    module_itself = import_module(module_path)
    if not hasattr(module_itself, class_name):
        raise ClassIncludeError(
            "The Python module '%s' has no '%s' class." % (module_path, class_name))
    return getattr(module_itself, class_name)

def include(arg, namespace=None, app_name=None):
    mod = None
    if isinstance(arg, basestring):
        try:
            ns = include_class(arg)()
        except ClassIncludeError:
            mod, include_namespace, include_app_name = django_include(
                arg, namespace=namespace, app_name=app_name)
        else:
            mod = modulize(ns)
    elif isinstance(arg, types.ModuleType):
        if arg.__name__ not in sys.modules:
            sys.modules.update({ arg.__name__: arg, })
        mod = arg
    elif isinstance(arg, Namespace):
        mod = modulize(arg)
    elif isinstance(arg, types.ClassType):
        mod = modulize(arg())
    return (mod, namespace, app_name)


class ArgumentSink(object):
    creation_counter = 0
    
    def __init__(self, *args, **kwargs):
        self.creation_counter = ArgumentSink.creation_counter
        ArgumentSink.creation_counter += 1
        self._args = args
        self._kwargs = kwargs
        self._options = {}
    
    def contribute_to_class(self, cls, name):
        self.name = name
        setattr(cls, name, self)
        if hasattr(cls, '_meta'):
            if hasattr(cls._meta, 'options'):
                self.process(name=name, **cls._meta.options)
    
    def process(self, **options):
        if hasattr(self, '_options'):
            self._options.update(options)
    
    def evaluate(self):
        return \
            tuple(self._args) + \
            tuple(self._kwargs.items())


class URLBase(ArgumentSink):
    def __init__(self, url_pattern, action, *args, **kwargs):
        self.url_pattern = url_pattern
        self.action = action
        super(URLBase, self).__init__(*args, **kwargs)
    
    def evaluate(self):
        return url(
            self.url_pattern, self.action, self._kwargs,
            name=self.name)


class View(URLBase):
    #@memo(CACHE)
    def ____evaluate(self):
        if isinstance(self.action, basestring):
            if isinstance(self._options.get('view_prefix'), basestring):
                if len(self._options.get('view_prefix')) > 0:
                    new_action = ".".join([self._options.get('view_prefix'), self.action])
                    self.action = new_action
        return super(View, self).evaluate()


class Redirect(URLBase):
    #@memo(CACHE)
    def evaluate(self):
        if isinstance(self.action, basestring):
            redirect_url = self.action
            self.action = redirect_to
            self._kwargs = dict(url=redirect_url)
        return super(Redirect, self).evaluate()


class ReverseRedirect(URLBase):
    #@memo(CACHE)
    def evaluate(self):
        if isinstance(self.action, basestring):
            redirect_url_name = self.action
            reverse_kw = dict()
            self.action = redirect_to
            if self._options.get('app_name'):
                reverse_kw.update({ 'current_app': self._options.get('app_name'), })
            if len(self._kwargs) > 0:
                reverse_kw.update({ 'kwargs': self._kwargs, })
            elif len(self._args) > 0:
                reverse_kw.update({ 'args': self._args, })
            self._kwargs = dict(url=reverse_lazy(redirect_url_name, **reverse_kw))
        return super(ReverseRedirect, self).evaluate()


class KeywordReverseRedirect(ReverseRedirect):
    def __init__(self, url_pattern, action, *args, **kwargs):
        kwre = re.compile(url_pattern)
        if kwre.groups > 0:
            for kw in kwre.groupindex.keys():
                kwargs.update({ kw: "%%(%s)s" % kw })
        super(KeywordReverseRedirect, self).__init__(url_pattern, action, *args, **kwargs)


class NamespaceOptions(object):
    def __init__(self, meta):
        self.view_prefix = getattr(meta, 'view_prefix', '')
        self.namespace = getattr(meta, 'namespace', '')
        self.app_name = getattr(meta, 'app_name', '')
    
    @property
    def options(self):
        return dict(
            view_prefix=self.view_prefix,
            namespace=self.namespace,
            app_name=self.app_name)
    
    def namespaced_name(self, name):
        if name.startswith(self.namespace.lower()):
            return name
        return "%s:%s" % (self.namespace.lower(), name.lower())
    
    def contribute_to_class(self, cls, name):
        cls._meta = self
        self.names = {}
        self.urls = []


class Namespacer(type):
    
    def __new__(cls, name, bases, attrs):
        super_new = super(Namespacer, cls).__new__
        parents = [b for b in bases if isinstance(b, Namespacer)]
        if not parents:
            return super_new(cls, name, bases, attrs)
        
        # Create the class.
        module = attrs.pop('__module__')
        new_class = super_new(cls, name, bases, { '__module__': module, })
        attr_meta = attrs.pop('Meta', None)
        if attr_meta:
            meta = attr_meta
        else:
            attr_meta = type('Meta', (object,), {})
            meta = getattr(new_class, 'Meta', None)
        
        new_class.add_to_class('_meta', NamespaceOptions(meta))
        new_class.add_to_class('Meta', attr_meta)
        
        for name, value in filter(lambda nv: isinstance(nv[1], ArgumentSink), attrs.items()):
            namespaced_name = new_class._meta.namespaced_name(name)
            new_class.add_to_class(name, value)
            new_class._meta.names[name] = namespaced_name
            new_class._meta.urls.append(
                (name, attrs.pop(name)),)
        
        # sort by creation_counter
        new_class._meta.urls.sort(
            key=lambda item: item[1].creation_counter)
        
        # Add all attributes to the class.
        for name, value in attrs.items():
            new_class.add_to_class(name, value)
        
        return new_class
    
    def add_to_class(cls, name, value):
        if hasattr(value, 'contribute_to_class'):
            value.contribute_to_class(cls, name)
        else:
            setattr(cls, name, value)


class Namespace(URLBase):
    __metaclass__ = Namespacer
    
    def __init__(self, url_pattern):
        super(Namespace, self).__init__(url_pattern, action=None)
    
    def process(self, **options):
        #if self.namespace and options.get('namespace', ''):
        if self.namespace:
            #parent_namespace = options.get('namespace', '')
            options.update({
                'namespace': self.namespace, })
            #options.update({
            #    'namespace': "%s:%s" % (
            #        parent_namespace, self.namespace), })
        if self.view_prefix and options.get('view_prefix', ''):
            parent_view_prefix = options.get('view_prefix', '')
            options.update({
                'view_prefix': "%s.%s" % (
                    parent_view_prefix, self.view_prefix), })
        self._meta.__dict__.update(options)
        super(Namespace, self).process(**options)
    
    @property
    #@memo(CACHE)
    def ordered_urls(self):
        return (getattr(self, uu[0]) for uu in self._meta.urls)
    
    @property
    #@memo(CACHE)
    def ordered_names(self):
        return (uu[0] for uu in self._meta.urls)
    
    @property
    def view_prefix(self):
        return getattr(self._meta, 'view_prefix',
            self._options.get('view_prefix', ''))
    @property
    def app_name(self):
        return getattr(self._meta, 'app_name',
            self._options.get('app_name', ''))
    @property
    def namespace(self):
        return getattr(self._meta, 'namespace',
            self._options.get('namespace', "dyn_ns_%s" % hash(self)))
    
    def _evaluate(self):
        urls_out = []
        for url_out in self.ordered_urls:
            urls_out.append(url_out.evaluate())
        return patterns(self.view_prefix, *urls_out)

    def evaluate(self):
        pprint(self.__dict__)
        #return url(self.url_pattern, (modulize(self), self._meta.namespace, self._meta.app_name), kwargs=self._kwargs)
        return url(self.url_pattern, (modulize(self), self._meta.namespace, self._meta.app_name), kwargs=self._kwargs)
    
    def connect(self, name):
        cls = self.__class__
        if hasattr(cls, '_meta'):
            if hasattr(cls._meta, 'options'):
                self.process(name=name, **cls._meta.options)
        return self.evaluate()
    
    def __dir__(self):
        return self.ordered_names
    
    @property
    def __members__(self):
        return self.__dir__()




# Django settings for the xx project.

DEBUG = True
TEMPLATE_DEBUG = DEBUG
ADMINS = ()
MANAGERS = ADMINS

import os
virtualpath = lambda *pths: os.path.join(os.path.dirname(os.path.dirname(__file__)), *pths)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': virtualpath('test', 'dev.db'),
        'TEST_NAME': virtualpath('test', 'test.db'),
        'USER': '',
        'PASSWORD': '',
        'HOST': '',
        'PORT': '',
    }
}

ROOT_URLCONF = 'test.urls'
WSGI_APPLICATION = 'test.wsgi.application'

TIME_ZONE = 'America/New_York'
LANGUAGE_CODE = 'en-us'
SITE_ID = 1
USE_I18N = False
USE_L10N = False
USE_TZ = False

MEDIA_ROOT = virtualpath('test', 'web', 'face')
MEDIA_URL = '/face/'
TEMPLATE_DIRS = ()
STATIC_ROOT = virtualpath('test', 'web', 'static')
STATIC_URL = '/static/'
STATICFILES_DIRS = ()
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'ceexe4vcdd2zety#noljtv7*1w1tthxldqp9s+4)m4r-)u^_ye'

TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.gzip.GZipMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.request",
    "django.core.context_processors.debug",
    #"django.core.context_processors.i18n", this is AMERICA
    "django.core.context_processors.media",
    "django.core.context_processors.static",
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.humanize',
    'django.contrib.messages',
    'django.contrib.markup',
    
    'django_extensions',
    'test_utils',
    
    'test',
    'testapp',
    'testapp.views',
)

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'root': {
        'level': 'INFO',
        'handlers': ['console'],
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s %(process)d %(thread)d %(message)s'
        },
    },
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        }
    },
    'loggers': {
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
        },
        'raven': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
        'sentry.errors': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
    },
}

TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

import platform
BASE_HOSTNAME = platform.node().lower()





from django.core.urlresolvers import reverse
#from django.core.urlresolvers import reverse_lazy
from django.conf.urls import patterns
from pprint import pprint
from url_namespaces import modulize
from url_namespaces import Namespace, View, Redirect, ReverseRedirect, KeywordReverseRedirect


class MyOtherURLs(Namespace):
        
    class Meta:
        namespace = 'testapp:views' # optional
        app_name = 'testapp' # optional
        view_prefix = '' # optional
        
    rss = View(
        r'^(?P<blog>[\w\-]+)/rss/?$',
            'testapp.views.blogrss',
            arg='yo', other_arg='dogg')
        
    index = View(
        r'^(?P<blog>[\w\-]+)/$',
            'testapp.views.blogpage')
        
    entry = View(
        r'^(?P<blog>[\w\-]+)/(?P<entry>[\w\-]+)/?$',
            'testapp.views.blogentrypage')
    
    
class MyURLs(Namespace):
        
    class Meta:
        namespace = 'testapp' # optional
        app_name = 'testapp' # optional
        view_prefix = '' # optional
        
    pages = MyOtherURLs(r'^/pages/')
        
    elsewhere = Redirect(
        r'^/yodogg?',
            "http://www.yodogg.com/")
        
    root = ReverseRedirect(
        r'^/?',
            'testapp:index', blog="test")
        
    entry_redirect = ReverseRedirect(
        r'^blog/(?P<blog>[\w\-]+)/(?P<entry>[\w\-]+)/?$',
            'testapp:entry', blog="%(blog)s", entry="%(entry)s")
        
    entry_redirect_again = KeywordReverseRedirect(
        r'^blog/(?P<blog>[\w\-]+)/(?P<entry>[\w\-]+)/?$',
            'testapp:entry')


urlpatterns = patterns('',
    MyURLs(r'^/?').connect('testapp'))



pprint(urlpatterns)


#md = modulize(ns)
#out = md.urlpatterns
#pprint(ns.evaluate())
#pprint(out)
#print ""

#pprint(ns._meta.__dict__)
#print ""
#pprint(ns.pages._meta.__dict__)
#print ""





#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#    URL-NAMESPACES -- Datastructures and syntax for saner Django URLConfs.
#
#    Copyright © 2012 Alexander Bohn
#
#    Permission is hereby granted, free of charge, to any person obtaining a copy 
#    of this software and associated documentation files (the "Software"), to deal 
#    in the Software without restriction, including without limitation the rights 
#    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
#    copies of the Software, and to permit persons to whom the Software is 
#    furnished to do so, subject to the following conditions:
#    
#    The above copyright notice and this permission notice shall be included in all 
#    copies or substantial portions of the Software.
#    
#    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
#    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
#    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
#    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
#    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
#    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
#    SOFTWARE.
#
from __future__ import print_function
import sys

name = 'django-url-namespaces'
version = '0.1.0'
packages = []
description = 'Declarative class-based datastructures, for simple and flexible URLConfs in Django.'
keywords = 'python django url urlconf config datastructures syntax'

classifiers = [
    'Development Status :: 5 - Production/Stable']

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if 'sdist' in sys.argv and 'upload' in sys.argv:
    import commands
    import os
    finder = "/usr/bin/find %s \( -name \*.pyc -or -name .DS_Store \) -delete"
    theplace = os.getcwd()
    if theplace not in (".", "/"):
        print("+ Deleting crapola from %s..." % theplace)
        print("$ %s" % finder % theplace)
        commands.getstatusoutput(finder % theplace)
        print("")

setup(
    name=name, version=version, description=description,
    keywords=keywords, platforms=['any'], packages=['url_namespaces'],
    
    author=u"Alexander Bohn", author_email='fish2000@gmail.com',
    
    license='MIT',
    url='http://github.com/fish2000/%s/' % name,
    download_url='http://github.com/fish2000/%s/zipball/master' % name,
    
    package_dir={
        'url_namespaces': 'url_namespaces' },
    
    install_requires=[
        'django'],
    
    classifiers=classifiers+[
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: OS Independent',
        'Operating System :: POSIX',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.6'],
)
